"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var node_fetch_1 = require("node-fetch");
var form_data_1 = require("form-data");
var webpack_log_1 = require("webpack-log");
var ElasticAPMSourceMapPlugin = /** @class */ (function () {
    function ElasticAPMSourceMapPlugin(config) {
        this.config = config;
    }
    ElasticAPMSourceMapPlugin.prototype.apply = function (compiler) {
        var _this = this;
        var logger = webpack_log_1.default({
            name: 'ElasticAPMSourceMapPlugin',
            level: this.config.logLevel || 'warn'
        });
        compiler.hooks.afterEmit.tapPromise('ElasticAPMSourceMapPlugin', function (compilation) {
            logger.debug("starting uploading sourcemaps with configs: " + JSON.stringify(_this.config) + ".");
            var chunks = compilation.getStats().toJson().chunks;
            return R.compose(function (promises) {
                return Promise.all(promises)
                    .then(function () { return logger.debug('finished uploading sourcemaps.'); })
                    .catch(function (err) {
                    logger.error(err);
                    throw err;
                });
            }, R.map(function (_a) {
                var sourceFile = _a.sourceFile, sourceMap = _a.sourceMap;
                var formData = new form_data_1.default();
                var bundleFilePath = _this.config.publicPath + "/" + sourceFile;
                formData.append('sourcemap', compilation.assets[sourceMap].source());
                formData.append('service_version', _this.config.serviceVersion);
                formData.append('bundle_filepath', bundleFilePath);
                formData.append('service_name', _this.config.serviceName);
                var headers = _this.config.secret
                    ? { Authorization: "Bearer " + _this.config.secret }
                    : undefined;
                return node_fetch_1.default(_this.config.serverURL, {
                    method: 'POST',
                    body: formData,
                    headers: headers
                })
                    .then(function (response) {
                    var responseText = response.text();
                    if (response.ok) {
                        logger.debug(responseText);
                        return responseText;
                    }
                    else {
                        var errMessage = "error while uploading " + sourceMap + " to Elastic APM. Response " + responseText;
                        logger.error(errMessage);
                        throw new Error(errMessage);
                    }
                })
                    .then(function () {
                    logger.debug("uploaded " + sourceMap + " to Elastic APM with bundle_filepath: " + bundleFilePath + ".");
                });
            }), R.map(function (_a) {
                var files = _a.files;
                var sourceFile = R.find(R.test(/\.js$/), files);
                var sourceMap = R.find(R.test(/\.js\.map$/), files);
                return { sourceFile: sourceFile, sourceMap: sourceMap };
            }))(chunks);
        });
    };
    return ElasticAPMSourceMapPlugin;
}());
exports.default = ElasticAPMSourceMapPlugin;
//# sourceMappingURL=elastic-apm-sourcemap-webpack-plugin.js.map